@@ -1,24 +1,24 @@
 /*******************************************************************
    Copyright (C) 2001-7 Leo Breiman, Adele Cutler and Merck & Co., Inc.
-  
+
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
- 
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.                            
+   GNU General Public License for more details.
 *******************************************************************/
 
 /******************************************************************
- * buildtree and findbestsplit routines translated from Leo's 
+ * buildtree and findbestsplit routines translated from Leo's
  * original Fortran code.
  *
  *      copyright 1999 by leo Breiman
- *      this is free software and can be used for any purpose. 
- *      It comes with no guarantee.  
+ *      this is free software and can be used for any purpose.
+ *      It comes with no guarantee.
  *
  ******************************************************************/
 #include <Rmath.h>
@@ -27,8 +27,8 @@
 
 void regTree(double *x, double *y, int mdim, int nsample, int *lDaughter,
              int *rDaughter,
-             double *upper, double *avnode, int *nodestatus, int nrnodes, 
-             int *treeSize, int nthsize, int mtry, int *mbest, int *cat,  
+             double *upper, double *avnode, int *nodestatus, int nrnodes,
+             int *treeSize, int nthsize, int mtry, int *mbest, int *cat,
 	     double *tgini, int *varUsed) {
     int i, j, k, m, ncur, *jdex, *nodestart, *nodepop;
     int ndstart, ndend, ndendl, nodecnt, jstat, msplit;
@@ -36,13 +36,13 @@ void regTree(double *x, double *y, int mdim, int nsample, int *lDaughter,
 
     nodestart = (int *) Calloc(nrnodes, int);
     nodepop   = (int *) Calloc(nrnodes, int);
-    
+
     /* initialize some arrays for the tree */
     zeroInt(nodestatus, nrnodes);
     zeroInt(nodestart, nrnodes);
     zeroInt(nodepop, nrnodes);
     zeroDouble(avnode, nrnodes);
-    
+
     jdex = (int *) Calloc(nsample, int);
     for (i = 1; i <= nsample; ++i) jdex[i-1] = i;
 
@@ -50,7 +50,7 @@ void regTree(double *x, double *y, int mdim, int nsample, int *lDaughter,
     nodestart[0] = 0;
     nodepop[0] = nsample;
     nodestatus[0] = NODE_TOSPLIT;
-    
+
     /* compute mean and sum of squares for Y */
     av = 0.0;
     ss = 0.0;
@@ -60,42 +60,42 @@ void regTree(double *x, double *y, int mdim, int nsample, int *lDaughter,
 	av = (i * av + d) / (i + 1);
     }
     avnode[0] = av;
-    
+
     /* start main loop */
     for (k = 0; k < nrnodes - 2; ++k) {
 	if (k > ncur || ncur >= nrnodes - 2) break;
 	/* skip if the node is not to be split */
-	if (nodestatus[k] != NODE_TOSPLIT) continue; 
-	
-	/* initialize for next call to findbestsplit */         
+	if (nodestatus[k] != NODE_TOSPLIT) continue;
+
+	/* initialize for next call to findbestsplit */
 	ndstart = nodestart[k];
 	ndend = ndstart + nodepop[k] - 1;
 	nodecnt = nodepop[k];
 	sumnode = nodecnt * avnode[k];
 	jstat = 0;
 	decsplit = 0.0;
-	
-	findBestSplit(x, jdex, y, mdim, nsample, ndstart, ndend, &msplit, 
-                      &decsplit, &ubest, &ndendl, &jstat, mtry, sumnode, 
+
+	findBestSplit(x, jdex, y, mdim, nsample, ndstart, ndend, &msplit,
+                      &decsplit, &ubest, &ndendl, &jstat, mtry, sumnode,
                       nodecnt, cat);
 	if (jstat == 1) {
 	    /* Node is terminal: Mark it as such and move on to the next. */
 	    nodestatus[k] = NODE_TERMINAL;
 	    continue;
-	} 
+	}
         /* Found the best split. */
         mbest[k] = msplit;
         varUsed[msplit - 1] = 1;
 	upper[k] = ubest;
 	tgini[msplit - 1] += decsplit;
 	nodestatus[k] = NODE_INTERIOR;
-	
+
 	/* leftnode no.= ncur+1, rightnode no. = ncur+2. */
 	nodepop[ncur + 1] = ndendl - ndstart + 1;
 	nodepop[ncur + 2] = ndend - ndendl;
 	nodestart[ncur + 1] = ndstart;
 	nodestart[ncur + 2] = ndendl + 1;
-    
+
 	/* compute mean and sum of squares for the left daughter node */
 	av = 0.0;
 	ss = 0.0;
@@ -110,7 +110,7 @@ void regTree(double *x, double *y, int mdim, int nsample, int *lDaughter,
 	if (nodepop[ncur + 1] <= nthsize) {
 	    nodestatus[ncur + 1] = NODE_TERMINAL;
 	}
-	
+
 	/* compute mean and sum of squares for the right daughter node */
 	av = 0.0;
 	ss = 0.0;
@@ -125,7 +125,7 @@ void regTree(double *x, double *y, int mdim, int nsample, int *lDaughter,
 	if (nodepop[ncur + 2] <= nthsize) {
 	    nodestatus[ncur + 2] = NODE_TERMINAL;
 	}
-	
+
 	/* map the daughter nodes */
 	lDaughter[k] = ncur + 1 + 1;
 	rDaughter[k] = ncur + 2 + 1;
@@ -145,8 +145,8 @@ void regTree(double *x, double *y, int mdim, int nsample, int *lDaughter,
 }
 
 /*--------------------------------------------------------------*/
-void findBestSplit(double *x, int *jdex, double *y, int mdim, int nsample, 
-		   int ndstart, int ndend, int *msplit, double *decsplit, 
+void findBestSplit(double *x, int *jdex, double *y, int mdim, int nsample,
+		   int ndstart, int ndend, int *msplit, double *decsplit,
 		   double *ubest, int *ndendl, int *jstat, int mtry,
 		   double sumnode, int nodecnt, int *cat) {
     int last, ncat[32], icat[32], lc, nl, nr, npopl, npopr;
@@ -169,7 +169,7 @@ void findBestSplit(double *x, int *jdex, double *y, int mdim, int nsample,
     critmax = 0.0;
     ubestt = 0.0;
     for (i=0; i < mdim; ++i) mind[i] = i;
-    
+
     last = mdim - 1;
     for (i = 0; i < mtry; ++i) {
 	critvar = 0.0;
@@ -210,7 +210,7 @@ void findBestSplit(double *x, int *jdex, double *y, int mdim, int nsample,
 	for (j = ndstart; j <= ndend; ++j) v[j] = xt[j];
 	for (j = 1; j <= nsample; ++j) ncase[j - 1] = j;
 	R_qsort_I(v, ncase, ndstart + 1, ndend + 1);
-	if (v[ndstart] >= v[ndend]) continue;	    
+	if (v[ndstart] >= v[ndend]) continue;
 	/* ncase(n)=case number of v nth from bottom */
 	/* Start from the right and search to the left. */
 	critParent = sumnode * sumnode / nodecnt;
@@ -247,7 +247,7 @@ void findBestSplit(double *x, int *jdex, double *y, int mdim, int nsample,
 	    }
 	}
     }
-    *decsplit = critmax; 
+    *decsplit = critmax;
 
     /* If best split can not be found, set to terminal node and return. */
     if (*msplit != -1) {
@@ -266,10 +266,10 @@ void findBestSplit(double *x, int *jdex, double *y, int mdim, int nsample,
                 nr++;
                 ncase[nr - 1] = jdex[j];
             }
-	    } 
-        if (*ndendl >= ndend) *ndendl = ndend - 1; 
+	    }
+        if (*ndendl >= ndend) *ndendl = ndend - 1;
         for (j = ndstart; j <= ndend; ++j) jdex[j] = ncase[j];
-	
+
         lc = cat[*msplit - 1];
         if (lc > 1) {
             for (j = 0; j < lc; ++j) {
@@ -284,24 +284,25 @@ void findBestSplit(double *x, int *jdex, double *y, int mdim, int nsample,
     Free(v);
     Free(yl);
     Free(xt);
-    Free(ut); 
+    Free(ut);
 }
 
 /*====================================================================*/
 void predictRegTree(double *x, int nsample, int mdim,
-		    int *lDaughter, int *rDaughter, int *nodestatus, 
-                    double *ypred, double *split, double *nodepred, 
+		    int *lDaughter, int *rDaughter, int *nodestatus,
+                    double *ypred, double *split, double *nodepred,
                     int *splitVar, int treeSize, int *cat, int maxcat,
                     int *nodex) {
-    int i, j, k, m, npack, *cbestsplit;
-    
+    int i, j, k, m, *cbestsplit;
+	unsigned int npack;
+
     /* decode the categorical splits */
     if (maxcat > 1) {
         cbestsplit = (int *) Calloc(maxcat * treeSize, int);
         zeroInt(cbestsplit, maxcat * treeSize);
         for (i = 0; i < treeSize; ++i) {
             if (nodestatus[i] != NODE_TERMINAL && cat[splitVar[i] - 1] > 1) {
-                npack = (int) split[i];
+                npack = (unsigned int) split[i];
                 /* unpack `npack' into bits */
                 for (j = 0; npack; npack >>= 1, ++j) {
                     cbestsplit[j + i*maxcat] = npack & 1;
@@ -315,7 +316,7 @@ void predictRegTree(double *x, int nsample, int mdim,
 	while (nodestatus[k] != NODE_TERMINAL) { /* go down the tree */
 	    m = splitVar[k] - 1;
 	    if (cat[m] == 1) {
-		k = (x[m + i*mdim] <= split[k]) ? 
+		k = (x[m + i*mdim] <= split[k]) ?
 		    lDaughter[k] - 1 : rDaughter[k] - 1;
 	    } else {
 	        /* Split by a categorical predictor */
